##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
"""Localiazer gettext Expression

implementation of TALES, with handlers for gettext.
"""
__version__='$Revision: 0.2 $'[11:-2]

import sys, types, traceback, re
from Products.PageTemplates import Expressions
from Products.PageTemplates.TALES import TALESError

_close_tag = re.compile(r'\</[a-z]*\>$', re.IGNORECASE)


class GettextExpr:
    """ gettext expression
    like <b tal:content="gettext:expr">Text</b>
    
    """

    def __init__(self, name, expr, engine):
        self._name = name
        self._expr = expr.strip()

    def __call__(self, econtext):
        lo = econtext.local_vars
        gl = econtext.global_vars
        lo_has_key = econtext.local_vars.has_key
	
        lang = None
        catalogs = None
        message = None
        add = 1

        if self._expr:
            message = econtext.evaluate(self._expr)
        else:
            # get content of html tag and use as message
            for t in econtext.tal_tagContent:
                if t[0] == 'rawtextOffset':
                    message = _close_tag.split(t[1][0])[0]
                    break
                if t[0] == 'rawtextColumn':
                    message = t[1][0]
                    break

        if not message: return ''

        message = message.strip()
                    
        # language
        if not lang:
            lang = gl['request'].user_lang

        try:
            # get list of MessageCatalog object
            if not catalogs:
                if lo_has_key('catalogs'):
                    catalogs = lo['catalogs']
                else:
                    catalogs = gl.get(
                        'catalogs',
                        getattr(econtext.contexts['here'], 'Messages'))

            if type(catalogs) != types.ListType: catalogs = [catalogs]

            # get message
            for catalog in catalogs:
                if catalog.hasLS(message):
                    result = catalog(message, lang)
                    break
            else:
                result = catalogs[0](message, lang, add)

            return result
        except:
            return message

    def __str__(self):
        return '%s expression %s' % (self._name, `self._expr`)

    def __repr__(self):
        return '%s:%s' % (self._name, `self._expr`)


class CatalogsExpr:
    """ catalogs expression """
    def __init__(self, name, expr, engine):
        self._name = name
        self._expr = expr.strip()

    def __call__(self, econtext):
        try:
            lo = econtext.local_vars
            gl_get = econtext.global_vars.get
            lo_has_key = econtext.local_vars.has_key

            if lo_has_key('catalogs'):
                catalogs = lo['catalogs']
            else:
                catalogs = gl_get('catalogs', [])

            if type(catalogs) != types.ListType:
                catalogs = [catalogs]

            try:
                val = econtext.evaluate(self._expr)
            except:
                val = econtext.evaluate('nocall:' + self._expr)

            lo['catalogs'] = [val] + catalogs

            return lo['catalogs']
        except:
            traceback.print_exc()
            return None
    
        
    def __str__(self):
        return '%s expression %s' % (self._name, `self._expr`)

    def __repr__(self):
        return '%s:%s' % (self._name, `self._expr`)


def initialize():
    engine = Expressions.getEngine()
    if not engine.getTypes().has_key('gettext'):
        engine.registerType('gettext', GettextExpr)
    if not engine.getTypes().has_key('catalogs'):
        engine.registerType('catalogs', CatalogsExpr)
