##############################################################################
# Copyright (C) 2001 Kim Nikolay <fafhrd@datacom.kz>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################

"""Portal Localizer tool."""
__version__='$Revision: 0.4 $'[11:-2]

import os, sys, Globals, traceback
from Globals import InitializeClass, DTMLFile
from webdav.common import rfc1123_date
from base64 import decodestring
from string import split
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from ZPublisher import BeforeTraverse
from Products.CMFCore.utils import UniqueObject, \
     getToolByName, SimpleItemWithProperties, _checkPermission
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.CMFCorePermissions import \
     ManagePortal, AccessContentsInformation, ModifyPortalContent, \
     View, ListFolderContents
from Products.Localizer.LanguageManager import LanguageManager
from utils import _dtmldir


class LocalizerTool(UniqueObject, SimpleItemWithProperties, LanguageManager,
                    ActionProviderBase):
    '''  '''
    id = 'portal_localizer'
    title = 'Portal Localizer'
    meta_type = 'Portal Localizer'

    _actions = []
    
    security = ClassSecurityInfo()

    manage_options = (
        ActionProviderBase.manage_options +
        ({ 'label' : 'Overview', 'action' : 'manage_overview'},) +
        LanguageManager.manage_options +
        SimpleItemWithProperties.manage_options
        )

    def __init__(self, languages=('en',)):
        self._languages = languages

    def manage_beforeDelete(self, item, container):
        """ """
        self.portal_actions.deleteActionProvider(self.id)

    def manage_afterAdd(self, item, container):
        """ """
        if self.id not in self.portal_actions.listActionProviders():
            self.portal_actions.addActionProvider(self.id)

    #
    #    ZMI methods
    #
    security.declareProtected( ManagePortal, 'manage_overview')
    manage_overview = DTMLFile( 'explainLocalizerTool', _dtmldir)

    def _get_first_lang(self, req):
        langs = self.get_languages()
        
        if req.cookies.has_key('planguage'):
            l = req.cookies['planguage']
            if l in langs: return l
                
        for l in req['USER_PREF_LANGUAGES']:
            if l in langs:
                return l
        return 'en'
        
    #security.declarePublic('getUserLanguage')
    #def getUserLanguage(self):
    #    try:
    #        return self.REQUEST.user_lang
    #    except:
    #        return self._get_first_lang(self.REQUEST)

    security.declarePublic('getContextLanguage')
    def getContextLanguage(self):
        lang = self.REQUEST.get('lang', '')
        if lang: return lang
        else:
            return self.REQUEST.user_lang

    security.declarePublic('setUserLanguage')
    def setUserLanguage(self, lang, REQUEST=None):
        request = self.REQUEST
        request.user_lang = lang

        if not self.portal_membership.isAnonymousUser():
            self.portal_registration.setProperties(language=lang)

        request.RESPONSE.setCookie(
            'planguage', lang, path='/',
            expires=rfc1123_date(DateTime('01-01-2021')))
        request.RESPONSE.setCookie(
            'planguage_changed', int(DateTime()), path='/',)

    security.declarePrivate('setContextLanguage')
    def setContextLanguage(self, lang):
        if lang in self.get_languages():
            self.REQUEST.context_lang = lang
        else:
            raise ValueError, 'Portal not support %s language'%lang

    #
    # for portal_actions
    #
    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """ Invoke by the portal_actions tool. """
        if info is None: return self._actions
        actions = []
        content = info.content
        if hasattr(content, 'aq_parent'):
            parent = content.aq_parent
            if (hasattr(aq_base(parent), '_isLocaleContent') and
                getattr(aq_base(parent), '_isLocaleContent') and
                content.getId() in self.get_languages()):
                for lang in self.get_languages_tuple():
                    if _checkPermission(View, parent._getOb(lang[0])):
                        actions.append(
                            {'name': lang[1],
                             'url': parent.absolute_url() + '/%s/view'%lang[0],
                             'permissions': [View],
                             'category': 'language'})
                folder = parent.aq_parent
                actions.append({
                    'name': 'Up folder',
                    'url': folder.absolute_url() + '/',
                    'permissions': [View],
                    'category': 'folder'})
                actions.append({
                    'name': 'Folder contents',
                    'url': folder.absolute_url() + '/folder_contents',
                    'permissions': [ListFolderContents],
                    'category': 'folder'})

        return actions + self._actions

    security.declarePublic('getIdFor')
    def getIdFor(self, content):
        """ """
        if hasattr(content, 'aq_parent'):
            parent = aq_base(content.aq_parent)
            if (hasattr(parent, '_isLocaleContent') and
                getattr(parent, '_isLocaleContent')):
                return parent.getId() + '(%s)'%content.getId()
        return content.getId()

    security.declarePublic('getLocaleContainer')
    def getLocaleContainer(self, content):
        """ return PortalLocaleContainer """
        if hasattr(content, 'aq_parent'):
            parent = content.aq_parent
            if (hasattr(aq_base(parent), '_isLocaleContent') and
                getattr(aq_base(parent), '_isLocaleContent')):
                return parent
        return content

    security.declarePublic('checkPermission')
    def checkPermission(self, permissionName, object, subobjectName=None):
        '''
        Checks whether the current user has the given permission on
        the given object or subobject.
        '''
        if subobjectName is not None:
            object = getattr(object, subobjectName)
        if hasattr(aq_base(object), '_isLocaleContent'):
            object = object.LocaleContent()
        try:
            return _checkPermission(permissionName, object)
        except:
            return 0

    def __call__(self, container=None, req=None):
        ''' The __before_publishing_traverse__ hook .'''
        if container is not None:
            try:
                lang = None

                # is this right ? req._auth
                name, passwd = tuple(split(
                    decodestring(split(req._auth)[-1]),':'))
                
                portal = self.portal_url.getPortalObject()
                u = portal.acl_users.getUser(name)
                if u is None:
                    u = portal.aq_parent.acl_users.getUser(name)
                if u is not None:
                    u = self.portal_membership.wrapUser(u)
                    lang = u.getProperty('language')
                
                if not lang:
                    lang = self._get_first_lang(req)

                if not req.cookies.has_key('planguage') or \
                       req.cookies['planguage'] != lang:
                    req.RESPONSE.setCookie(
                        'planguage', lang, path='/',
                        expires=rfc1123_date(DateTime('01-01-2021')))
                    req.RESPONSE.setCookie(
                        'planguage_changed', int(DateTime()), path='/',)

                req.user_lang = lang
                req.context_lang = lang
            except:
                req.user_lang = self._get_first_lang(req)
                req.context_lang = req.user_lang

    def manage_beforeDelete(self, item, container):
        if item is self:
            handle = self.meta_type + '/' + self.getId()
            BeforeTraverse.unregisterBeforeTraverse(container, handle)

    def manage_afterAdd(self, item, container):
        """ """
        if item is self:
            handle = self.meta_type + '/' + self.getId()
            container = container.this()
            nc = BeforeTraverse.NameCaller(self.getId())
            BeforeTraverse.registerBeforeTraverse(container, nc, handle, 180)

InitializeClass(LocalizerTool)
