# -#-   Python    -#- 
#
# Authors: Nik Kim <fafhrd@datacom.kz> 
# File: PortalLocaleContent.py
# Created: Tue Nov 27 15:10:22 2001
#
__version__='$Revision: 0.4 $'[11:-2]

import os, Globals, traceback, copy, string, new
from Globals import InitializeClass, Persistent, DTMLFile
from Acquisition import aq_base, aq_parent
from OFS.ObjectManager import ObjectManager
from ZODB.PersistentMapping import PersistentMapping
from AccessControl import getSecurityManager, ClassSecurityInfo
from Products.CMFCore.utils import getToolByName, _getViewFor, _checkPermission
from Products.CMFCore.DynamicType import DynamicType
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import \
     View, ManageProperties, AddPortalContent, AddPortalFolders, \
     ModifyPortalContent
from utils import _dtmldir

factory_type_information = (
    { 'id'             : 'LocaleContent'
      , 'meta_type'      : 'LocaleContent'
      , 'description'    : """ Locale Content ."""
      , 'icon'           : 'folder_icon.gif'
      , 'product'        : 'CMFLocalizer'
      , 'factory'        : 'manage_addLocaleContent'
      , 'immediate_view' : ''
      , 'filter_content_types' : 1
      , 'allowed_content_types': ['Document','News Item','Link']
      , 'actions'        :
      ( { 'name'       : 'View'
          , 'action'     : ''
          , 'permissions': (View,)
          },
        )
      },
    )

_marker = []
class PortalLocaleContent( PortalFolder ):
    """ """
    meta_type = 'LocaleContent'
    portal_type = 'LocaleContent'
    
    security = ClassSecurityInfo()

    isPortalContent = 1
    isPrincipiaFolderish = 0
    _isLocaleContent = 1
    _needReindex = 1
    
    def __init__(self, id, title='', type_name='Document'):
        self.type_name = type_name
        PortalFolder.__init__(self, id, title)

    def getReviewState(self, *args, **kw):
        """ get review_state of locale content """
        return self.portal_workflow.getInfoFor(self.LocaleContent(),
                                               'review_state', '')

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, stx_level=None, setlevel=0):
        """ mimic """
        return self.LocaleContent().CookedBody(stx_level, setlevel)

    #
    # DublinCore
    #
    security.declareProtected(View, 'Type')
    def Type(self):
        """ return mimic type """
        return self.type_name

    security.declareProtected(View, 'Title')
    def Title( self ):
        """ return mimic title """
        for key in self.__dict__.keys():
            try:
                if key[-11:] == '_Permission':
                    del self.__dict__[key]
            except: pass
            
        lo_con = self.LocaleContent(self.REQUEST.user_lang)
        title = lo_con.Title()
        if title:
            return title
        else:
            return self.title_or_id()
        
    security.declarePublic( 'Date' )
    def Date( self ):
        "Dublin Core element - default date"
        return self.LocaleContent().Date()

    security.declarePublic( 'Creator' )
    def Creator( self ):
        """ return creator """
        return self.LocaleContent().Creator()

    security.declarePublic( 'Description' )
    def Description( self ):
        """ return mimic description """
        return self.LocaleContent().Description()

    security.declarePublic('LocaleTitle')
    def LocaleTitle( self ):
        """ return mimic title """
        titles = []
        for lang in self.portal_localizer.get_languages():
            lo_con = self.LocaleContent(lang)
            if self.portal_workflow.getInfoFor(
                lo_con, 'review_state', '') =='published':
                titles.append(lo_con.Title())
        return titles

    security.declarePublic('LocaleDescription')
    def LocaleDescription( self ):
        """ return mimic title """
        descs = []
        for lang in self.portal_localizer.get_languages():
            lo_con = self.LocaleContent(lang)
            if self.portal_workflow.getInfoFor(
                lo_con, 'review_state', '') =='published':
                descs.append(lo_con.Description())
        return descs

    security.declarePublic( 'Language' )
    def Language(self):
        """ """
        langs = []
        for lang in self.portal_localizer.get_languages():
            if self.portal_workflow.getInfoFor(
                self[lang], 'review_state','') == 'published':
                langs.append(lang)
        return langs

    def getIcon(self):
        """ return mimic type icon """
        return self.portal_types.getTypeInfo(self.type_name).getIcon()
    icon = getIcon

    def _getOb(self, id, default=_marker):
        """ return object by id """
        try:
            return ObjectManager._getOb(self, id)
        except:
            if id in self.portal_localizer.get_languages():
                self.invokeFactory(self.type_name, id, title=self.title)
                obj = ObjectManager._getOb(self, id)
                if obj.Type() == 'Document':
                    obj.__dict__['text'] = ''
                    obj.__dict__['cooked_text'] = ''

                if obj.Language() != id:
                    obj.setLanguage(id)

                return obj
            else:
                raise

    def objectIds(self, spec=None):
        """ return contained objects """
        return self.portal_localizer.get_languages()

    security.declareProtected(View, 'LocaleContent')
    def LocaleContent(self, lang=None, REQUEST=None):
        """ return content for current or specified language """
        #prepare lang
        if not lang: lang = self.portal_localizer.getContextLanguage()

        lng_obj = self._getOb(lang)

        if lng_obj.Language() != lang:
            lng_obj.setLanguage(lang)

        return lng_obj.__of__(self)

    def __getitem__(self, name):
        """ """
        if name == 'Content':
            return self.LocaleContent()
        else:
            return PortalFolder.__getitem__(self, name)

    def __getattr__(self, name):
        """ """
        parent = Globals.get_request()['PARENTS'][1]
        if name[-11:] == '_Permission':
            try:
                return getattr(self.LocaleContent(
                    parent.portal_localizer.getContextLanguage()), name)
            except:
                raise
        else:
            #return getattr(parent, name)
            #return
            raise AttributeError, name

    def __setattr__(self, name, value):
        try:
            if name[-11:] == '_Permisssion': return
        except: pass
        self.__dict__[name] = value
            

    #security.declareProtected(ModifyPortalContent, 'editLocaleMetadata')
    #def editLocaleMetadata(self, caller_id, **kwargs):
    #    """ set metadata to all locale content """
    #    if self._recurse: return
    #    self._recurse = 1
    #    for id, obj in self.objectItems():
    #        if id != caller_id:
    #            if _checkPermission(ModifyPortalContent, obj):
    #                obj.metadata_edit(**kwargs)
    #    self._recurse = 0

    #
    # View
    #
    def __call__(self, container=None, request=None):
        """ Invoke the default view. """
        try:
            return self.LocaleContent().view()
        except:
            traceback.print_exc()
            return None


    security.declareProtected(View, 'view')
    view = __call__

    index_html = None


    security.declareProtected(AddPortalContent, 'manage_addLocaleContent')
    def manage_addLocaleContent(self, id, title='',
                                lo_type_name=None, REQUEST=None):
        """Add a new PortalLocaleContent object
        """
        ob=PortalLocaleContent(id, title, lo_type_name)
        self._setObject(id, ob)
        if REQUEST is not None:
            return self.folder_contents( # XXX: ick!
                self,REQUEST,portal_status_message="PortalLocaleContent added")


manage_addLocaleContent = PortalLocaleContent.manage_addLocaleContent
